/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/import_obj.h>
#include<vcg/complex/algorithms/update/topology.h>
#include<vcg/complex/algorithms/update/normal.h>
#include<map>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Color4b, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};


int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;
  int loadmask;
  vcg::tri::io::ImporterOFF<MyMesh>::LoadMask(argv[1],loadmask);
  
  if(vcg::tri::io::ImporterOFF<MyMesh>::Open(m,argv[1],loadmask)!=vcg::tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }
  
  if(loadmask & vcg::tri::io::Mask::IOM_VERTCOLOR)
    printf("File has per vertex color\n");
  

  if(vcg::tri::HasPerVertexColor(m))
    printf("mesh has per vertex color\n");
  
        
  vcg::tri::RequirePerVertexNormal(m);
  vcg::tri::UpdateNormal<MyMesh>::PerVertexNormalized(m);
  
  vcg::tri::UpdateTopology<MyMesh>::FaceFace(m);
  std::map<std::pair<MyVertex *,MyVertex *>, int > nonManifMap;
  for(size_t i =0; i<m.face.size(); ++i)
  {
    for(int j=0;j<3;++j)
    {
      if(m.face[i].FFp(j) == &(m.face[i]) )
        printf("face %i on edge %i is border\n",i,j);
      else
      {
        MyFace *fp = &(m.face[i]);
        MyFace *fpp = fp->FFp(j);
        int     fpi=  fp->FFi(j);
        
        if(fp != fpp->FFp(fpi))  
        {
          printf("face %i on edge %i is non manifold\n",i,j);
          MyVertex *v0 = fp->V0(j); // this is exactly the same of fp->V(j) just prettier.
          MyVertex *v1 = fp->V1(j); //MyVertex *v1 = fp->V((j+1)%3);
          if(v0>v1) std::swap(v0,v1);
          nonManifMap[std::make_pair(v0,v1)]++;           
        }
      }
    }
    printf("Found %i non manifold edges\n",nonManifMap.size());
  }
  
  printf("Input mesh  vn:%i fn:%i\n",m.VN(),m.FN());
  printf( "Mesh has %i vert and %i faces\n", m.VN(), m.FN() );

  return 0;
}
